// Macho - Machine Objects for C++
//
// This class library supports a subset of the UML statechart notation
// (hierarchical states, entry and exit actions, state histories,
// state data) in straight C++, similar in spirit to the GoF "State"
// design pattern.
//
// Copyright (c) 2005 by Eduard Hiti (feedback to macho@gmx.org)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//
// You are encouraged to provide any changes, extensions and corrections for
// this software to the author at the above-mentioned email address for
// inclusion into future versions.
//
//
// Description:
//
// States are represented as C++ classes. The hierarchy of states follows the
// inheritance relation between these state classes. A set of state classes for
// a single state machine derives directly or indirectly from a top state class,
// making it the composite state holding all other states. Events are processed
// by calling virtual methods of the top state class. Substates redefine the
// behaviour of these event handler methods.
//
// Special methods "entry", "exit" and "init" are called on state entry, state
// exit and state initialization of super- and substates (in the order defined
// by statechart semantics and current machine state).
//
// An object of type "Machine" maintains the current state of a state machine
// and dispatches events to it. The "Machine" type is a template class
// parametrized with the top state class of the state machine to be run.
//
// State data is not kept in state classes (because state class instances are
// created just once and then reused, whereas state data should be instantiated
// or destroyed each time its state is entered or left). State data is put in
// "Box" types specific to each state class instead, which are managed by the
// Machine object. States may be parametrized with preinitialized boxes on
// Machine object creation or state transition. Boxes are retrieved by calling
// the "box" method. Superstate boxes are accessible by qualifiying the "box"
// method with the state class name (e.g. TOP::box()).
//
// History of previous substates can be kept for superstates. Upon direct
// transition into the superstate the previous substate is then entered. History
// can be shallow (only direct substates) or deep (any substate).
//
//
// Example:
//
// #include "Macho.hpp"
// #include <iostream>
// using namespace std;
//
// namespace Example {
//  TOPSTATE(Top) {
//      struct Box {
//          Box() : myData(0) {}
//          Box(long l) : myData(l) {}
//          long myData;
//      };
//
//      STATE(Top)
//
//      virtual void event1() {}
//      virtual void event2() {}
//
//  private:
//      void entry();
//      void exit();
//      void init();
//  };
//
//  SUBSTATE(Super, Top) {
//      STATE(Super)
//      HISTORY()
//
//  private:
//      void entry();
//      void exit();
//  };
//
//  SUBSTATE(StateA, Super) {
//      struct Box {
//          Box() : myData(0) {}
//          Box(int i) : myData(i) {}
//          int myData;
//      };
//
//      STATE(StateA)
//
//      void event1();
//
//   private:
//      void entry();
//      void exit();
//
//  };
//
//  SUBSTATE(StateB, Super) {
//      STATE(StateB)
//
//      void event2();
//
//  private:
//      void entry();
//      void exit();
//  };
//
//  void Top::entry() { cout << "Top::entry" << endl; }
//  void Top::exit() { cout << "Top::exit" << endl; }
//  void Top::init() { setState<StateA>(new StateA::Box(42)); }
//
//  void Super::entry() { cout << "Super::entry" << endl; }
//  void Super::exit() { cout << "Super::exit" << endl; }
//
//  void StateA::entry() { cout << "StateA::entry" << endl; }
//  void StateA::exit() { cout << "StateA::exit" << endl; }
//  void StateA::event1() { setState<StateB>(); }
//
//  void StateB::entry() { cout << "StateB::entry" << endl; }
//  void StateB::exit() { cout << "StateB::exit" << endl; }
//  void StateB::event2() { setState<StateA>(); }
// }
//
// int main() {
//  Macho::Machine<Example::Top> m(new Example::Top::Box(11));
//  m->event1();
//  m->event2();
//
//  return 0;
// }
//
// Output is:
//
// Top::entry
// Super::entry
// StateA::entry
// StateA::exit
// StateB::entry
// StateB::exit
// StateA::entry
// StateA::exit
// Super::exit
// Top::exit
//
//
// Version History:
//
//      0.9 (released 2006-01-15):
//          - Introduction of queuable event type
//
//      0.8.2 (released 2005-12-15):
//          - Code size reduction by minimizing use of template classes
//
//      0.8.1 (released 2005-12-1):
//          - Added MSVC6 variant
//          - Added method "clearHistoryDeep"
//
//      0.8 (released 2005-11-1):
//          - Initial release
//

#ifndef __MACHO_HPP__
#define __MACHO_HPP__

#include <cassert>
#include <vector>

#ifdef __MACHO_TRACE__
#   include <iostream>
#endif


////////////////////////////////////////////////////////////////////////////////
// Various macros for state and history declaration

// Use this macro to define your top state class.
#define TOPSTATE(TOP) \
  struct TOP : public Macho::Link< TOP, Macho::Root< TOP > >

// Use this macro for all your other state classes.
#define SUBSTATE(STATE, SUPERSTATE) \
  struct STATE : public Macho::Link< STATE, SUPERSTATE >

// Use this macro in your class definition to give it state functionality
// (mandatory). If you have a state box declare it BEFORE macro invocation!
#define STATE(S) \
protected: \
    friend class Macho::Machine<TOP>; \
    /* Get to your Box with this method: */ \
    inline Box & box() { return *static_cast<Box *>(_box()); } \
public: \
    /* Constructor and destructor already defined: you can't (and shouldn't) have your own! */ \
    /* For you a state class "constructor" and "destructor" are its entry and exit method! */ \
    S(Macho::MachineBase * machine) : Macho::Link<S, SUPER>(machine) { \
        /* Compile time check: S must derive directly from Link<S, SUPER> */ \
        Macho::Link<S, SUPER> * p = 0; \
        _LINK * p2 = 0; \
        p = p2; \
        p2 = p; \
    } \
    ~S() {} \
    static const char * _state_name() { return #S; }

// Use this macro to select deep history conservation.
#define DEEPHISTORY() \
private: \
    /* If no superstate has history, SUPER::_setHistorySuper is a NOOP */ \
    void _saveHistory(Macho::StateInfo * self, Macho::StateInfo * shallow, Macho::StateInfo * deep) \
    { self->setHistory(deep); SUPER::_setHistorySuper(self, deep); } \
protected: \
    /* Substates may use _setHistorySuper to bubble up history */ \
    void _setHistorySuper(Macho::StateInfo * self, Macho::StateInfo * deep) \
    { self->setHistorySuper(deep); } \
public:

// Use this macro to select shallow history conservation.
#define HISTORY() \
private: \
    /* If no superstate has history, SUPER::_setHistorySuper is NOOP */ \
    void _saveHistory(Macho::StateInfo * self, Macho::StateInfo * shallow, Macho::StateInfo * deep) \
    { self->setHistory(shallow); SUPER::_setHistorySuper(self, deep); } \
protected: \
    /* Substates may use _setHistorySuper to bubble up history */ \
    void _setHistorySuper(Macho::StateInfo * self, Macho::StateInfo * deep) \
    { self->setHistorySuper(deep); } \
public:


////////////////////////////////////////////////////////////////////////////////
// Everything else is put into namespace Macho.
namespace Macho {

// Some methods are prefixed with an underscore to prevent name clashes with
// deriving classes.

// Helper functions for tracing.
#ifdef __MACHO_TRACE__
    inline void MACHO_TRC1(const char * msg) { std::cout << msg << std::endl; }
    inline void MACHO_TRC2(const char * state, const char * msg)
    { std::cout << "State " << state << ": " << msg << std::endl; }
    inline void MACHO_TRC3(const char * state, const char * msg1, const char * msg2)
    { std::cout << "State " << state << ": " << msg1 << " " << msg2 << std::endl; }
#else
#   define MACHO_TRC1(MSG)
#   define MACHO_TRC2(STATE, MSG)
#   define MACHO_TRC3(STATE, MSG1, MSG2)
#endif

    typedef unsigned int Key;

    class MachineBase;

    template<typename T>
    class Machine;

    template<typename T>
    class IEvent;

    class State;
    class StateInfo;

    template<class T>
    void * createBox(void * & place);

    ////////////////////////////////////////////////////////////////////////////////
    // Box for states which don't declare own Box class.
    class EmptyBox {
        EmptyBox() {}
    public:
        static EmptyBox theEmptyBox;
    };

    EmptyBox EmptyBox::theEmptyBox;

    ////////////////////////////////////////////////////////////////////////////////
    // Helper functions for box creation
    template<class T>
    void * createBox(void * & place) {
        if (!place)
            place = ::operator new(sizeof(T));

        new (place) T;

        void * box = place;
        place = 0;

        return box;
    }

    template<class T>
    inline void deleteBox(void * & box, void * & place) {
        assert(box);
        assert(!place);

        static_cast<T *>(box)->~T();
        place = box;
        box = 0;
    }

    // Specializations for EmptyBox: EmptyBox object gets reused over and over
    // and never is deleted.
    template<>
    void * createBox<EmptyBox>(void * & place) {
        return &EmptyBox::theEmptyBox;
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Base class for all state classes.
    class State {
    public:
        virtual ~State() {}

        // Get unique name of state.
        static Key key() {
            return 0;
        }

    protected:
        State(MachineBase * machine)
            : _myMachine(machine)
        {}

        // Initiate transition to a new state.
        // Template parameter S is the new state to enter.
        // Transition is performed AFTER control flow returns to the Machine object.
        // Initiating more than one transition is considered an error!
        // The new state may receive a preinitialized box:
        // setState<StateA>(new StateA::Box("someData"));
        template<typename S>
        void setState(typename S::Box * box = 0);

        // Same as setState, ignores history however.
        template<typename S>
        void setStateDirect(typename S::Box * box = 0);

        // This is the method to bubble up history information
        // for states whose superstates have no history (so does nothing).
        virtual void _setHistorySuper(StateInfo * self, StateInfo * deep) {}

    private:
        // State exit. Not allowed to initiate state change.
        virtual void exit() {}

        // State entry. Not allowed to initiate state change.
        virtual void entry() {}

        // Special kind of state entry: Upon transition to a new state,
        // entry methods of that state and its superstates are called;
        // 'init' however is called only on the one state the transition
        // actually goes to.
        // Is allowed to change state.
        virtual void init() {}

    private:
        // for _myMachine
        template<typename C, typename P>
        friend class Link;

        // for _myMachine
        template<class T>
        friend class Root;

        friend class StateInfo;

        friend class MachineBase;

        // Create StateInfo object of state.
        static StateInfo * _getInfo(MachineBase * machine);

        // Default history strategy (no history).
        virtual void _saveHistory(StateInfo * self, StateInfo * shallow, StateInfo * deep) {}

    private:
        MachineBase * _myMachine;
    };

    ////////////////////////////////////////////////////////////////////////////////
    // Each state machine has its own name generator. The generator returns unique
    // identifiers. The identifiers are consecutive integers starting from zero,
    // which allows use as index into a vector for fast access.
    // Root always has key zero.
    template<class TOP>
    Key newKey() {
        static Key counter = 1;
        return counter++;
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Base class for the top state (and indirectly of all others).
    // This is the "real" top state. By entering state 'Root' we trigger entry
    // and exit actions of the top state.
    template<typename T>
    class Root : public State {
    public:
        // This typedef is an alias for the top state in all (sub)states.
        typedef T TOP;

    protected:
        Root(MachineBase * machine)
            // Get StateInfo object from machine.
            : State(machine)
        {}

        void dispatch(IEvent<TOP> * event);
    };

    ////////////////////////////////////////////////////////////////////////////////
    // This class links substates to superstates by deriving from the superstate
    // and being derived from by the substate.
    // Substates inherit event handlers from superstates for reuse or redefinition
    // this way.
    template<typename Child, typename Parent>
    class Link : public Parent {
    public:
        // Alias for superstate.
        typedef Parent SUPER;

        // Alias for topstate.
        typedef typename Parent::TOP TOP;

        // Default box type.
        typedef EmptyBox Box;

        // Get unique name of state.
        static Key key() {
            return _myName;
        }

        // Is machine m in this state?
        static bool isCurrent(Machine<TOP> & m) {
            return m.myCurrentState->isChild(_getInfo(&m));
        }

        // Is machine m in exactly this state?
        static bool isCurrentDirect(Machine<TOP> & m) {
            return m.myCurrentState == _getInfo(&m);
        }

        static void clearHistory(Machine<TOP> & m) { _getInfo(&m)->setHistory(0); }
        static void clearHistoryDeep(Machine<TOP> & m) { _getInfo(&m)->clearHistoryDeep(); }

    protected:
        // Needed to perform compile time checks.
        typedef Link<Child, Parent> _LINK;

        Link(MachineBase * machine)
            : Parent(machine)
            , _myStateInfo(0)
        {}

        // These definitions seem redundant but they are not!
        // They override parent definitions so each substate gets either
        // this default or their own, but never its parents definitions.
        virtual void entry() {}
        virtual void init() {}
        virtual void exit() {}

        // This method keeps '_myStateInfo' attribute private.
        void * _box();

    private:
        // for _getInfo
        template<typename C, typename P>
        friend class Link;

        // for _getInfo
        friend class State;

        // for _getInfo
        friend class Machine<TOP>;

        // Create StateInfo object of state.
        static StateInfo * _getInfo(MachineBase * machine);

        // Default history strategy (no history). Not redundant!
        void _saveHistory(StateInfo * self, StateInfo * shallow, StateInfo * deep) {
            // Bubble up history. If no superstate has history, _setHistorySuper will do nothing.
            this->_setHistorySuper(self, deep);
        }

    private:
        // StateInfo object of this class.
        StateInfo * _myStateInfo;

        static Key _myName;
    };

    template<class C, class P> Key Link<C, P>::_myName = newKey<typename P::TOP>();

    ////////////////////////////////////////////////////////////////////////////////
    // StateInfo describes a state. Keeps history, box and state object for state.
    // StateInfo object is created the first time state is entered.
    // There is at most one StateInfo object per state per machine instance in
    // existence.
    class StateInfo {
    public:
        StateInfo(MachineBase * machine, StateInfo * parent)
            : myMachine(machine)
            , myHistory(0)
            , myInstance(0)
            , myParent(parent)
            , myBox(0)
            , myBoxPlace(0)
        {
            assert(machine);
            if (parent)
                parent->addSubstate(this);
        }

        virtual ~StateInfo() {
            assert(myBox == 0 || myBox == &EmptyBox::theEmptyBox);

            delete myInstance;
            if (myBoxPlace)
                ::operator delete(myBoxPlace);
        }

        // Perform entry actions.
        // 'first' is true on very first call.
        void entry(StateInfo * previous, bool first = true);

        // Perform exit actions.
        void exit(StateInfo * next);

        // Perform init action.
        void init(bool history);

        void saveHistory(StateInfo * shallow, StateInfo * deep) {
            // Check state's history strategy.
            instance()->_saveHistory(this, shallow, deep);
        }

        // Update superstate's history information:
        void setHistorySuper(StateInfo * deep) {
            if (this->myParent)
                // Let it choose between deep or shallow history.
                this->myParent->saveHistory(this, deep);
        }

        // Box has been created explicitly.
        void setBox(void * box) {
            assert(!this->myBox);

            if (myBoxPlace) {
                ::operator delete(myBoxPlace);
                myBoxPlace = 0;
            }

            this->myBox = box;
        }

        virtual void createInstance() {
            this->myInstance = new State(myMachine);
        }

        virtual void createBox() {}
        virtual void deleteBox() {}

        virtual const char * name() const { return "Root"; }

        // Is 'state' a superstate?
        inline bool isChild(StateInfo * state) const {
            return this == state || (myParent && myParent->isChild(state));
        }

        inline State * instance() {
            if (!this->myInstance)
                createInstance();
            return this->myInstance;
        }

        inline void * box() {
            assert(myBox);
            return myBox;
        }

        inline MachineBase * machine() {
            return myMachine;
        }

        void setHistory(StateInfo * history) {
            myHistory = history;
        }

        void clearHistoryDeep() {
            setHistory(0);

            typedef std::vector<StateInfo *>::iterator iter;
            for(iter it = mySubstates.begin(); it != mySubstates.end(); ++it)
                (*it)->clearHistoryDeep();
        }

        void addSubstate(StateInfo * state) {
            mySubstates.push_back(state);
        }

    protected:
        MachineBase * myMachine;
        StateInfo * myHistory;
        State * myInstance;   // Instance of state class
        StateInfo * myParent;
        std::vector<StateInfo *> mySubstates;
        void * myBox;
        void * myBoxPlace;  // Reused box memory
    };

    // Perform entry actions.
    // 'first' is true on very first call.
    void StateInfo::entry(StateInfo * previous, bool first) {
        // Only Root has no parent
        if (!myParent)
            return;

        // first entry or previous state is not substate -> perform entry
        if (first || ! previous->isChild(this)) {
            this->myParent->entry(previous, false);

            // Could be set from outside (or EmptyBox)
            if (!this->myBox)
                createBox();

            MACHO_TRC2(name(), "Entry");
            instance()->entry();
        }
    }

    // Perform exit actions.
    void StateInfo::exit(StateInfo * next) {
        // Only Root has no parent
        if (!myParent)
            return;

        // self transition or next state is not substate -> perform exit
        if (next == this || ! next->isChild(this)) {
            MACHO_TRC2(name(), "Exit");
            instance()->exit();

            // EmptyBox should be most common box, so keep it
            if (this->myBox != &EmptyBox::theEmptyBox)
                deleteBox();

            this->myParent->exit(next);
        }
    }

    ////////////////////////////////////////////////////////////////////////////////
    // StateInfo for substates (including Top ;-)
    // Has methods to create state specific objects.
    template<class T>
    struct SubStateInfo : public StateInfo
    {
        typedef typename T::Box Box;

        SubStateInfo(MachineBase * machine, StateInfo * parent)
            : StateInfo(machine, parent)
        {}

        virtual const char * name() const { return T::_state_name(); }

        virtual void createInstance() {
            this->myInstance = new T(myMachine);
        }

        virtual void createBox() {
            this->myBox = Macho::createBox<Box>(myBoxPlace);
        }

        virtual void deleteBox() {
            Macho::deleteBox<Box>(myBox, myBoxPlace);
        }
    };

    ////////////////////////////////////////////////////////////////////////////////
    // Definitions for queuable event types

    // Generic interface for event objects (available only to MachineBase)
    class IEventBase {
    public:
        virtual ~IEventBase() {}
        virtual void dispatch(StateInfo *) = 0;
    };

    // Interface for event objects (bound to a top state)
    template<class TOP>
    class IEvent : protected IEventBase {
        friend class Machine<TOP>;
        friend class Root<TOP>;
    };

    // Event with four parameters
    template<class TOP, class R, class P1, class P2, class P3, class P4>
    class Event4 : public IEvent<TOP> {
        typedef R (TOP::*Signature)(P1, P2, P3, P4);

    public:
        Event4(Signature handler, P1 p1, P2 p2, P3 p3, P4 p4)
            : myHandler(handler)
            , myParam1(p1)
            , myParam2(p2)
            , myParam3(p3)
            , myParam4(p4)
        {}

    protected:
        void dispatch(StateInfo * info) {
            TOP * state = static_cast<TOP *>(info->instance());
            (state->*myHandler)(myParam1, myParam2, myParam3, myParam4);
        }

        Signature myHandler;
        P1 myParam1;
        P2 myParam2;
        P3 myParam3;
        P4 myParam4;
    };

    // Event with three parameters
    template<class TOP, class R, class P1, class P2, class P3>
    class Event3 : public IEvent<TOP> {
        typedef R (TOP::*Signature)(P1, P2, P3);

    public:
        Event3(Signature handler, P1 p1, P2 p2, P3 p3)
            : myHandler(handler)
            , myParam1(p1)
            , myParam2(p2)
            , myParam3(p3)
        {}

    protected:
        void dispatch(StateInfo * info) {
            TOP * state = static_cast<TOP *>(info->instance());
            (state->*myHandler)(myParam1, myParam2, myParam3);
        }

        Signature myHandler;
        P1 myParam1;
        P2 myParam2;
        P3 myParam3;
    };

    // Event with two parameters
    template<class TOP, class R, class P1, class P2>
    class Event2 : public IEvent<TOP> {
        typedef R (TOP::*Signature)(P1, P2);

    public:
        Event2(R (TOP::*handler)(P1, P2), P1 p1, P2 p2)
            : myHandler(handler)
            , myParam1(p1)
            , myParam2(p2)
        {}

    protected:
        void dispatch(StateInfo * info) {
            TOP * state = static_cast<TOP *>(info->instance());
            (state->*myHandler)(myParam1, myParam2);
        }

        Signature myHandler;
        P1 myParam1;
        P2 myParam2;
    };

    // Event with one parameter
    template<class TOP, class R, class P1>
    class Event1 : public IEvent<TOP> {
        typedef R (TOP::*Signature)(P1);

    public:
        Event1(R (TOP::*handler)(P1), P1 p1)
            : myHandler(handler)
            , myParam1(p1)
        {}

    protected:
        void dispatch(StateInfo * info) {
            TOP * state = static_cast<TOP *>(info->instance());
            (state->*myHandler)(myParam1);
        }

        Signature myHandler;
        P1 myParam1;
    };

    // Event with no parameters
    template<class TOP, class R>
    class Event0 : public IEvent<TOP> {
        typedef R (TOP::*Signature)();

    public:
        Event0(R (TOP::*handler)())
            : myHandler(handler)
        {}

    protected:
        void dispatch(StateInfo * info) {
            TOP * state = static_cast<TOP *>(info->instance());
            (state->*myHandler)();
        }

        Signature myHandler;
    };

    // Event creating functions using type inference
    template<class TOP, class R, class P1, class P2, class P3, class P4>
    inline IEvent<TOP> * Event(R (TOP::*handler)(P1, P2, P3, P4), P1 p1, P2 p2, P3 p3, P4 p4) {
        return new Event4<TOP, R, P1, P2, P3, P4>(handler, p1, p2, p3, p4);
    }

    template<class TOP, class R, class P1, class P2, class P3>
    inline IEvent<TOP> * Event(R (TOP::*handler)(P1, P2, P3), P1 p1, P2 p2, P3 p3) {
        return new Event3<TOP, R, P1, P2, P3>(handler, p1, p2, p3);
    }

    template<class TOP, class R, class P1, class P2>
    inline IEvent<TOP> * Event(R (TOP::*handler)(P1, P2), P1 p1, P2 p2) {
        return new Event2<TOP, R, P1, P2>(handler, p1, p2);
    }

    template<class TOP, class R, class P1>
    inline IEvent<TOP> * Event(R (TOP::*handler)(P1), P1 p1) {
        return new Event1<TOP, R, P1>(handler, p1);
    }

    template<class TOP, class R>
    inline IEvent<TOP> * Event(R (TOP::*handler)()) {
        return new Event0<TOP, R>(handler);
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Base class for Machine objects.
    class MachineBase {
    protected:
        MachineBase()
            : myCurrentState(0)
            , myPendingState(0)
            , myPendingBox(0)
            , myPendingHistory(false)
            , myPendingEvent(0)
            , myStates(20, (StateInfo *) 0)
        {}

        ~MachineBase() {
            shutdown();

            // Free all StateInfo objects now.
            typedef std::vector<StateInfo *>::iterator iter;
            iter end = myStates.end();
            for (iter it = myStates.begin(); it != end; ++it) {
                delete *it;
            }

            delete myPendingEvent;
        }

        // Transition to new state (with optional preinitialized box).
        void setState(StateInfo * info, bool history, void * box) {
            setPendingState(info, history, box);
            rattleOn();
        }

        // Prepare transition to new state (performed on call to "rattleOn").
        // There can be only one state transition pending (asserts otherwise)!
        // "box" is an optional preinitialized state box for the new state.
        void setPendingState(StateInfo * info, bool history, void * box) {
            assert( (!myPendingState || myPendingState == info)
                    && "There is already a state transition pending!");

            myPendingState = info;
            myPendingBox = box;
            myPendingHistory = history;
        }

        void setPendingEvent(IEventBase * event) {
            assert(!myPendingEvent && "There is already an event pending!");

            myPendingEvent = event;
        }

        // Starts the machine. Will make it go into top state.
        // Optional parameter "box" is a preinitialized state box for the top
        // state.
        void start(StateInfo * info, void * box) {
            MACHO_TRC1("Starting Machine");

            myCurrentState = State::_getInfo(this);
            setState(info, false, box);
        }

        // Shuts machine down. Will exit any states and free all allocated
        // resources.
        void shutdown() {
            assert(!myPendingState);

            MACHO_TRC1("Shutting down Machine");

            // Performs exit actions.
            setState(State::_getInfo(this), false, 0);

            myCurrentState = 0;
        }

        // Performs pending state transition.
        void rattleOn();

        // Get StateInfo object for key.
        StateInfo * & getInfo(Key name) {
            if (name >= myStates.size())
                myStates.resize(name + 10, (StateInfo *) 0);

            return myStates[name];
        }

    protected:
        // for getInfo
        template<typename C, typename P>
        friend class Link;

        // for setPendingEvent
        template<class T>
        friend class Root;

        // for getInfo
        friend class State;

        // for setPendingState
        friend class StateInfo;

        // Current state of Machine object.
        StateInfo * myCurrentState;

        // Information about pending state transition.
        StateInfo * myPendingState;
        void * myPendingBox;
        bool myPendingHistory;

        IEventBase * myPendingEvent;

        // StateInfo objects.
        std::vector<StateInfo *> myStates;
    };

    // Performs pending state transition.
    void MachineBase::rattleOn() {
        while (myPendingState || myPendingEvent) {
            // Loop here because init actions might change state again.
            while (myPendingState) {
                MACHO_TRC3(myCurrentState->name(), "Transition to", myPendingState->name());

                // Entry/Exit actions may not dispatch events: set dummy event.
                if (!myPendingEvent)
                    myPendingEvent = (IEventBase *) &myPendingEvent;

                // Perform exit actions (which exactly depends on new state).
                myCurrentState->exit(myPendingState);

                // Store history information for previous state now.
                // Previous state will be used for deep history.
                myCurrentState->setHistorySuper(myCurrentState);

                StateInfo * previous = myCurrentState;
                myCurrentState = myPendingState;

                if (myPendingBox)
                    myCurrentState->setBox(myPendingBox);

                // Perform entry actions (which exactly depends on previous state).
                myCurrentState->entry(previous);

                // State transition complete.
                // Clear 'pending' information just now so that setState would assert in exits and entries, but not in init.
                myPendingState = 0;
                myPendingBox = 0;
                bool history = myPendingHistory;
                myPendingHistory = false;

                // "init" may change state again.
                myCurrentState->init(history);

                assert("Init may only transition to proper substates" &&
                       (!myPendingState ||
                        (myPendingState->isChild(myCurrentState)
                         && (myCurrentState != myPendingState))));

                // Clear dummy event if need be
                if (myPendingEvent == (IEventBase *) &myPendingEvent)
                    myPendingEvent = 0;
            }

            if (myPendingEvent) {
                IEventBase * event = myPendingEvent;
                myPendingEvent = 0;
                event->dispatch(myCurrentState);
                delete event;
            }
        }
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Implementation for StateInfo.
    void StateInfo::init(bool history) {
        if (history && this->myHistory) {
            MACHO_TRC3(name(), "History transition to", this->myHistory->name());
            this->myMachine->setPendingState(this->myHistory, true, 0);
        } else {
            assert(this->myInstance);
            MACHO_TRC2(name(), "Init");
            instance()->init();
        }

        this->myHistory = 0;
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Implementation for State

    // Create StateInfo object of state.
    inline StateInfo * State::_getInfo(MachineBase * machine) {
        // Look first in machine for existing StateInfo.
        StateInfo * & info = machine->getInfo(key());
        if (!info)
            info = new StateInfo(machine, 0);
        return info;
    }

    template<typename S>
    inline void State::setState(typename S::Box * box) {
        StateInfo * info = S::_getInfo(_myMachine);
        _myMachine->setPendingState(info, true, box);
    }

    // Same as setState, ignores history however.
    template<typename S>
    inline void State::setStateDirect(typename S::Box * box) {
        StateInfo * info = S::_getInfo(_myMachine);
        _myMachine->setPendingState(info, false, box);
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Implementation for Root
    template<class T>
    inline void Root<T>::dispatch(IEvent<TOP> * event) {
        _myMachine->setPendingEvent(event);
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Implementation for Link

    // Create StateInfo object of state.
    template<typename C, typename P>
    StateInfo * Link<C, P>::_getInfo(MachineBase * machine) {
        // Look first in machine for existing StateInfo.
        StateInfo * & info = machine->getInfo(key());
        if (!info)
            // Will create parent StateInfo object if not already existing.
            info = new SubStateInfo<C>(machine, P::_getInfo(machine));
        return info;
    }

    // This method keeps '_myStateInfo' attribute private.
    template<typename C, typename P>
    inline void * Link<C, P>::_box() {
        if (!_myStateInfo)
            _myStateInfo = this->_myMachine->getInfo(key());
        return _myStateInfo->box();
    }

    ////////////////////////////////////////////////////////////////////////////////
    // A Machine object maintains a current state.
    // The state can be any substate of template parameter TOP.
    // TOP is the Machine's top most and inital state. TOP must be defined by
    // the macro TOPSTATE. Event processing is done by calling methods (event
    // handlers) on the current state.
    // This is realized by defining an arrow ("->") operator on Machine,
    // forwarding to the interface of TOP.
    // Every possible event handler to be called must therefore appear in the
    // interface of TOP. Events are dispatched by using this operator on a
    // Machine object (e.g. "machine->event()").
    template<typename TOP>
    class Machine : public MachineBase {
    public:
        ////////////////////////////////////////////////////////////////////////////////
        // This class performs an action in its destructor after an event
        // handler has finished. Comparable to an After Advice in AOP.
        struct AfterAdvice {
            AfterAdvice(Machine<TOP> * m) : myMachine(m) {}

            // Event handler has finished execution. Execute pending transitions now.
            ~AfterAdvice() { myMachine->rattleOn(); }

            // this arrow operator finally dispatches to TOP interface.
            inline TOP * operator->() {
                return static_cast<TOP *>(myMachine->myCurrentState->instance());
            }

        private:
            Machine<TOP> * myMachine;
        };

        // Constructor takes an optional preinitialized Box parameter for its
        // initial (top) state.
        Machine(typename TOP::Box * box = 0)
        {
            // Compile time check: TOP must directly derive from Root<TOP>
            Root<TOP> * p = 0;
            typename TOP::SUPER *p2 = 0;
            p = p2;
            p2 = p;

            start(TOP::_getInfo(this), box);
        }

        // Don't return pointer to interface right now: we need to know when the
        // event handler has finished; return an AfterAdvice object instead:
        // it allows us to perform actions after access.
        AfterAdvice operator->() {
            assert(myCurrentState);
            assert(!myPendingState);

            MACHO_TRC2(myCurrentState->name(), "Dispatching event");

            // We need to know when the event handler has finished.
            return AfterAdvice(this);
        }

        void dispatch(IEvent<TOP> * event) {
            event->dispatch(myCurrentState);
            delete event;
            rattleOn();
        }

        // Allow (const) access to top state's box (for state data extraction).
        const typename TOP::Box & box() const {
            assert(myCurrentState);
            return static_cast<TOP *>(myCurrentState->instance())->TOP::box();
        }
    };

} // namespace Macho

#endif
