// Macho - Machine Objects for C++
//
// This class library supports a subset of the UML statechart notation
// (hierarchical states, entry and exit actions, state histories,
// state data) in straight C++, similar in spirit to the GoF "State"
// design pattern.
//
// Copyright (c) 2005 by Eduard Hiti (feedback to macho@gmx.org)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//
// You are encouraged to provide any changes, extensions and corrections for
// this software to the author at the above-mentioned email address for
// inclusion into future versions.
//
//
// Description:
//
// States are represented as C++ classes. The hierarchy of states follows the
// inheritance relation between these state classes. A set of state classes for
// a single state machine derives directly or indirectly from a top state class,
// making it the composite state holding all other states. Events are processed
// by calling virtual methods of the top state class. Substates redefine the
// behaviour of these event handler methods.
//
// Special methods "entry", "exit" and "init" are called on state entry, state
// exit and state initialization of super- and substates (in the order defined
// by statechart semantics and current machine state).
//
// An object of type "Machine" maintains the current state of a state machine
// and dispatches events to it. The "Machine" type is a template class
// parametrized with the top state class of the state machine to be run.
//
// State data is not kept in state classes (because state class instances are
// created just once and then reused, whereas state data should be instantiated
// or destroyed each time its state is entered or left). State data is put in
// "Box" types specific to each state class instead, which are managed by the
// Machine object. States may be parametrized with preinitialized boxes on
// Machine object creation or state transition. Boxes are retrieved by calling
// the "box" method. Superstate boxes are accessible by qualifiying the "box"
// method with the state class name (e.g. TOP::box()).
//
// History of previous substates can be kept for superstates. Upon direct
// transition into the superstate the previous substate is then entered. History
// can be shallow (only direct substates) or deep (any substate).
//
//
// Example:
//
// #include "Macho.hpp"
// #include <iostream>
// using namespace std;
//
// namespace Example {
//  TOPSTATE(Top) {
//      struct Box {
//          Box() : myData(0) {}
//          Box(long l) : myData(l) {}
//          long myData;
//      };
//
//      STATE(Top)
//
//      virtual void event1() {}
//      virtual void event2() {}
//
//  private:
//      void entry();
//      void exit();
//      void init();
//  };
//
//  SUBSTATE(Super, Top) {
//      STATE(Super)
//      HISTORY()
//
//  private:
//      void entry();
//      void exit();
//  };
//
//  SUBSTATE(StateA, Super) {
//      struct Box {
//          Box() : myData(0) {}
//          Box(int i) : myData(i) {}
//          int myData;
//      };
//
//      STATE(StateA)
//
//      void event1();
//
//   private:
//      void entry();
//      void exit();
//
//  };
//
//  SUBSTATE(StateB, Super) {
//      STATE(StateB)
//
//      void event2();
//
//  private:
//      void entry();
//      void exit();
//  };
//
//  void Top::entry() { cout << "Top::entry" << endl; }
//  void Top::exit() { cout << "Top::exit" << endl; }
//  void Top::init() { setState<StateA>(new StateA::Box(42)); }
//
//  void Super::entry() { cout << "Super::entry" << endl; }
//  void Super::exit() { cout << "Super::exit" << endl; }
//
//  void StateA::entry() { cout << "StateA::entry" << endl; }
//  void StateA::exit() { cout << "StateA::exit" << endl; }
//  void StateA::event1() { setState<StateB>(); }
//
//  void StateB::entry() { cout << "StateB::entry" << endl; }
//  void StateB::exit() { cout << "StateB::exit" << endl; }
//  void StateB::event2() { setState<StateA>(); }
// }
//
// int main() {
//  Macho::Machine<Example::Top> m(new Example::Top::Box(11));
//  m->event1();
//  m->event2();
//
//  return 0;
// }
//
// Output is:
//
// Top::entry
// Super::entry
// StateA::entry
// StateA::exit
// StateB::entry
// StateB::exit
// StateA::entry
// StateA::exit
// Super::exit
// Top::exit
//
//
// Version History:
//
//      0.8.1 (2005-12-1):
//          - Added MSVC6 variant
//          - Added method "clearHistoryDeep"
//
//      0.8 (2005-11-1):
//          - Initial release
//

#ifndef __MACHO_HPP__
#define __MACHO_HPP__

#include <cassert>
#include <vector>

#ifdef __MACHO_TRACE__
#   include <iostream>
#endif

#define TYPENAME


////////////////////////////////////////////////////////////////////////////////
// Various macros for state and history declaration

// Use this macro to define your top state class.
#define TOPSTATE(TOP) \
  struct TOP : public Macho::Link< TOP, Macho::Root< TOP > >

// Use this macro for all your other state classes.
#define SUBSTATE(STATE, SUPERSTATE) \
  struct STATE : public Macho::Link< STATE, SUPERSTATE >

// Use this macro in your class definition to give it state functionality
// (mandatory). If you have a state box declare it BEFORE macro invocation!
#define STATE(S) \
public: \
    /* In case methods entry, exit and init are private: */ \
    friend struct Macho::SubStateInfo<S>; \
    /* Lets machine access (top) box: */ \
    /* friend class Macho::Machine<S::TOP>; */ \
    /* Get to your Box with this method: */ \
    Box & box() { return *static_cast<Box *>(_box()); } \
    /* Constructor and destructor already defined: you can't (and shouldn't) have your own! */ \
    /* For you a state class "constructor" and "destructor" are its entry and exit method! */ \
    S(Macho::Machine<TOP> * machine) : Macho::Link<S, SUPER>(machine) { \
        /* Compile time check: S must derive directly from Link<S, SUPER> */ \
        Macho::Link<S, SUPER> * p = 0; \
        _LINK * p2 = 0; \
        p = p2; \
        p2 = p; \
    } \
    ~S() {} \
    static const char * _state_name() { return #S; }

// Use this macro to select deep history conservation.
#define DEEPHISTORY() \
public: \
    /* If no superstate has history, SUPER::_setHistorySuper is a NOOP */ \
    static inline void _saveHistory(Macho::StateInfo<TOP> * self, Macho::StateInfo<TOP> * shallow, Macho::StateInfo<TOP> * deep) \
    { self->setHistory(deep); SUPER::_setHistorySuper(self, deep); } \
    /* Substates may use _setHistorySuper to bubble up history */ \
    static inline void _setHistorySuper(Macho::StateInfo<TOP> * self, Macho::StateInfo<TOP> * deep) \
    { self->setHistorySuper(deep); } \
public:

// Use this macro to select shallow history conservation.
#define HISTORY() \
public: \
    /* If no superstate has history, SUPER::_setHistorySuper is NOOP */ \
    static inline void _saveHistory(Macho::StateInfo<TOP> * self, Macho::StateInfo<TOP> * shallow, Macho::StateInfo<TOP> * deep) \
    { self->setHistory(shallow); SUPER::_setHistorySuper(self, deep); } \
    /* Substates may use _setHistorySuper to bubble up history */ \
    static inline void _setHistorySuper(Macho::StateInfo<TOP> * self, Macho::StateInfo<TOP> * deep) \
    { self->setHistorySuper(deep); } \
public:


////////////////////////////////////////////////////////////////////////////////
// Everything else is put into namespace Macho.
namespace Macho {

// Some methods are prefixed with an underscore to prevent name clashes with
// deriving classes.

// Helper functions for tracing.
#ifdef __MACHO_TRACE__
    inline void _TRC1(const char * msg) { std::cout << msg << std::endl; }
    inline void _TRC2(const char * state, const char * msg)
    { std::cout << "State " << state << ": " << msg << std::endl; }
    inline void _TRC3(const char * state, const char * msg1, const char * msg2)
    { std::cout << "State " << state << ": " << msg1 << " " << msg2 << std::endl; }
#else
#  define _TRC1(MSG)
#  define _TRC2(STATE, MSG)
#  define _TRC3(STATE, MSG1, MSG2)
#endif

    typedef unsigned int Key;

    template<typename T>
    class Machine;

    template<typename T>
    class Root;

    ////////////////////////////////////////////////////////////////////////////////
    // Box for states which don't declare own Box class.
    struct EmptyBox {
    };

    ////////////////////////////////////////////////////////////////////////////////
    // StateInfo describes a state. Keeps history, box and state object for state.
    // Root<TOP> is described by StateInfo, every other state by SubStateInfo.
    // StateInfo object is created the first time state is entered.
    // There is at most one StateInfo object per state per machine instance in
    // existence.
    template<class TOP>
    struct StateInfo {
        StateInfo(Macho::Machine<TOP> * machine, StateInfo<TOP> * parent)
            : myMachine(machine)
            , myHistory(0)
            , myInstance(0)
            , myParent(parent)
            , myBox(0)
        {
            assert(machine);
        }

        virtual ~StateInfo() {
            assert(!myBox);
            delete myInstance;
        }

        virtual void entry(StateInfo<TOP> *, bool first = true, TOP * instance = 0) {}
        virtual void exit(StateInfo<TOP> *, TOP * instance = 0) {}
        virtual void init(bool history) {}
        virtual void setBox(void *) { assert(false); }
        virtual void createInstance(Machine<TOP> * machine) {}
        virtual const char * name() const { return "Root"; }
        virtual Key key() { return Root<TOP>::key(); }
        virtual void setHistorySuper(StateInfo<TOP> * deep) {}

        // Is 'state' a superstate?
        bool isChild(StateInfo<TOP> * state) const {
            return this == state || (myParent && myParent->isChild(state));
        }

        inline TOP * instance() {
            return this->myInstance;
        }

        // Check if given state class instance is valid.
        // If not, use own instance object.
        inline TOP * checkInstance(TOP * instance) {
            if (instance)
                return instance;

            if (!myInstance)
                createInstance(myMachine);
            return myInstance;
        }

        inline void * box() {
            assert(myBox);
            return myBox;
        }

        inline Machine<TOP> * machine() {
            return myMachine;
        }

        inline void setHistory(StateInfo<TOP> * history) {
            myHistory = history;
        }

        void clearHistoryDeep() {
            setHistory(0);

            typedef typename std::vector<StateInfo<TOP> *>::iterator iter;
            for(iter it = mySubstates.begin(); it != mySubstates.end(); ++it)
                (*it)->clearHistoryDeep();
        }

        void addSubstate(StateInfo<TOP> * state) {
            mySubstates.push_back(state);
        }

    protected:
        Machine<TOP> * myMachine;
        StateInfo<TOP> * myHistory;
        TOP * myInstance;   // Instance of state class
        StateInfo<TOP> * myParent;
        std::vector<StateInfo<TOP> *> mySubstates;
        void * myBox;
    };

    ////////////////////////////////////////////////////////////////////////////////
    // StateInfo for substates (including Top ;-)
    // Substates have superstates: that makes SubStateInfo different from StateInfo.
    template<class T>
    struct SubStateInfo : public StateInfo<typename T::TOP>
    {
        typedef typename T::TOP TOP;
        typedef typename T::Box Box;

        SubStateInfo(Machine<TOP> * machine, StateInfo<TOP> * parent)
            : StateInfo<TOP>(machine, parent)
        {
            parent->addSubstate(this);
        }

        ~SubStateInfo() {
        }

        virtual const char * name() const { return T::_state_name(); }

        // Update superstate's history information:
        // Let it choose between deep or shallow history.
        virtual void setHistorySuper(StateInfo<TOP> * deep) {
            typedef typename T::SUPER S;

            // Check superstate's history strategy.
            S::_saveHistory(this->myParent, this, deep);
        }

        virtual void createInstance(Machine<TOP> * machine) {
            this->myInstance = new T(machine);
        }

        // Box has been created explicitly.
        virtual void setBox(void * box) {
            assert(!this->myBox);
            this->myBox = box;
        }

        // Perform entry actions. Use 'instance' as state class object.
        // 'first' is true on very first call.
        virtual void entry(StateInfo<TOP> * last, bool first, TOP * instance) {
            // first entry or previous state is not substate -> perform entry
            if (first || ! last->isChild(this)) {
                instance = checkInstance(instance);

                this->myParent->entry(last, false, instance);

                // Could be set from outside
                if (!this->myBox)
                    this->myBox = new typename T::Box();

                _TRC2(T::_state_name(), "Entry");
                static_cast<T *>(instance)->T::entry();
            }
        }

        // Perform exit actions. Use 'instance' as state class object.
        virtual void exit(StateInfo<TOP> * next, TOP * instance) {
            // self transition or next state is not substate -> perform exit
            if (next == this || ! next->isChild(this)) {
                instance = checkInstance(instance);

                _TRC2(T::_state_name(), "Exit");
                static_cast<T *>(instance)->T::exit();

                delete static_cast<typename T::Box *>(this->myBox);
                this->myBox = 0;

                this->myParent->exit(next, instance);
            }
        }

        virtual void init(bool history) {
            if (history && this->myHistory) {
                _TRC3(T::_state_name(), "History transition to", this->myHistory->name());
                this->myMachine->setPendingState(this->myHistory, true, 0);
            } else {
                assert(this->myInstance);
                _TRC2(T::_state_name(), "Init");
                static_cast<T *>(this->myInstance)->T::init();
            }

            this->myHistory = 0;
        }

        virtual Key key() { return T::key(); }
    };

    ////////////////////////////////////////////////////////////////////////////////
    // Each state machine has its own name generator. The generator returns unique
    // identifiers. The identifiers are consecutive integers starting from zero,
    // which allows use as index into a vector for fast access.
    template<class TOP>
    Key newKey() {
        static Key counter = 0;
        return counter++;
    }

    template<class TOP>
    struct FactoryInitializer {
        typedef StateInfo<TOP> * (*InfoFactory)(Macho::Machine<TOP> * machine);
        FactoryInitializer(InfoFactory f, Key k) {
            if (k >= getFactories().size())
                getFactories().resize(k + 10, (InfoFactory) 0);

            getFactories()[k] = f;
        }
        void dummy() {} // MSVC6 removes static members if they are not used. WTF!

        static std::vector<InfoFactory> & getFactories() {
            static std::vector<InfoFactory> factories;
            return factories;
        }
    };

    ////////////////////////////////////////////////////////////////////////////////
    // Base class for the top state (and indirectly of all others).
    // This is the "real" top state. By entering state 'Root' we trigger entry
    // and exit actions of the top state.
    template<typename T>
    class Root {
    public:
        // This typedef is an alias for the top state in all (sub)states.
        typedef T TOP;

        // Get unique name of state.
        static Key key() {
            static Key k = newKey<TOP>();
            return k;
        }

    protected:
        Root(Macho::Machine<TOP> * machine)
            // Get StateInfo object from machine.
            : _myState(machine->getInfo(key()))
        {
            assert(_myState);
            _init_factory.dummy();  // WTF! MSVC6, leave my static members alone!
        }

        virtual ~Root() {}

        // State exit. Not allowed to initiate state change.
        void exit() {}

        // State entry. Not allowed to initiate state change.
        void entry() {}

        // Special kind of state entry: Upon transition to a new state,
        // entry methods of that state and its superstates are called;
        // init however is called only on the one state the transition
        // actually goes to.
        // Is allowed to change state.
        void init() {}

        // Initiate transition to a new state.
        // Template parameter S is the new state to enter.
        // Transition is performed AFTER control flow returns to the Machine object.
        // Initiating more than one transition is considered an error!
        // The new state may receive a preinitialized box:
        // setState<StateA>(new StateA::Box("someData"));
        void setState(Key key, void * box = 0) {
            typedef StateInfo<TOP> * (*InfoFactory)(Macho::Machine<TOP> * machine);
            InfoFactory f = FactoryInitializer<TOP>::getFactories()[key];
            StateInfo<TOP> * info = f(_myState->machine());
            _myState->machine()->setPendingState(info, true, box);
        }

        // Same as setState, ignores history however
        void setStateDirect(Key key, void * box = 0) {
            typedef StateInfo<TOP> * (*InfoFactory)(Macho::Machine<TOP> * machine);
            InfoFactory f = FactoryInitializer<TOP>::getFactories()[key];
            StateInfo<TOP> * info = f(_myState->machine());
            _myState->machine()->setPendingState(info, false, box);
        }

    public:
        // Default history strategy (no history).
        static inline void _saveHistory(StateInfo<TOP> * self, StateInfo<TOP> * shallow, StateInfo<TOP> * deep) {}

    protected:
        // This is the method to bubble up history information
        // for states whose superstates have no history (so does nothing).
        static inline void _setHistorySuper(StateInfo<TOP> * self, StateInfo<TOP> * deep) {}

    private:
        // for _getInfo
        //template<typename C, typename P>
        //friend class Link;

        // for _getInfo
        friend class Machine<TOP>;

    protected:
        // Create StateInfo object of state.
        static StateInfo<TOP> * _getInfo(Macho::Machine<TOP> * machine) {
            // Look first in machine for existing StateInfo.
            StateInfo<TOP> * & info = machine->getInfo(key());
            if (!info)
                info = new StateInfo<TOP>(machine, 0);
            return info;
        }

    private:
        static FactoryInitializer<TOP> _init_factory;

        // StateInfo object of this state.
        StateInfo<TOP> * _myState;
    };

    template<class TOP> FactoryInitializer<TOP> Root<TOP>::_init_factory = FactoryInitializer<TOP>(&Root<TOP>::_getInfo, Root<TOP>::key());

    ////////////////////////////////////////////////////////////////////////////////
    // This class links substates to superstates by deriving from the superstate
    // and being derived from by the substate.
    // Substates inherit event handlers from superstates for reuse or redefinition
    // this way.
    template<typename Child, typename Parent>
    class Link : public Parent {
    public:
        // Alias for superstate.
        typedef Parent SUPER;

        // Alias for topstate.
        typedef typename Parent::TOP TOP;

        // Default box type.
        typedef EmptyBox Box;

        // Get unique name of state.
        static Key key() {
            static Key k = newKey<TOP>();
            return k;
        }

        // Is machine m in this state?
        static bool isCurrent(Machine<TOP> & m) {
            return m.myCurrentState->isChild(_getInfo(&m));
        }

        // Is machine m in exactly this state?
        static bool isCurrentDirect(Machine<TOP> & m) {
            return m.myCurrentState == _getInfo(&m);
        }

        static void clearHistory(Machine<TOP> & m) { _getInfo(&m)->setHistory(0); }
        static void clearHistoryDeep(Machine<TOP> & m) { _getInfo(&m)->clearHistoryDeep(); }

    protected:
        // Needed to perform compile time checks.
        typedef Link<Child, Parent> _LINK;

        Link(Macho::Machine<TOP> * machine)
            : Parent(machine)
            // Get StateInfo object from machine.
            , _myState(static_cast<SubStateInfo<Child> *>(machine->getInfo(key())))
        {
            assert(_myState);
            _init_factory.dummy();  // WTF! MSVC6, leave my static members alone!
        }

        // These definitions seem redundant but they are not!
        // They override parent definitions so each substate gets either
        // this default or their own, but never its parents definitions.
        void entry() {}
        void init() {}
        void exit() {}

    public:
        // Default history strategy (no history). Not redundant!
        static inline void _saveHistory(StateInfo<TOP> * self, StateInfo<TOP> * shallow, StateInfo<TOP> * deep) {
            // Bubble up history. If no superstate has history, this does nothing.
            _setHistorySuper(self, deep);
        }

    protected:
        // This method keeps '_myState' attribute private.
        void * _box() { return _myState->box(); }

    private:
        // for _getInfo
        //template<typename C, typename P>
        //friend class Link;

        // for _getInfo
        friend class Machine<TOP>;

        // for _getInfo
        friend class Root<TOP>;

    protected:
        // Create StateInfo object of state.
        static StateInfo<TOP> * _getInfo(Macho::Machine<TOP> * machine) {
            // Look first in machine for existing StateInfo.
            StateInfo<TOP> * & info = machine->getInfo(key());
            if (!info)
                // Will create parent StateInfo object if not already existing.
                info = new SubStateInfo<Child>(machine, Parent::_getInfo(machine));
            return info;
        }

    private:
        static FactoryInitializer<TOP> _init_factory;

        // StateInfo object of this class.
        SubStateInfo<Child> * _myState;
    };

    template<class C, class P> FactoryInitializer<TYPENAME P::TOP> Link<C, P>::_init_factory = FactoryInitializer<TYPENAME P::TOP>(&Link<C, P>::_getInfo, Link<C, P>::key());

    ////////////////////////////////////////////////////////////////////////////////
    // A Machine object maintains a current state.
    // The state can be any substate of template parameter TOP.
    // TOP is the Machine's top most and inital state. TOP must be defined by
    // the macro TOPSTATE. Event processing is done by calling methods (event
    // handlers) on the current state.
    // This is realized by defining an arrow ("->") operator on Machine,
    // forwarding to the interface of TOP.
    // Every possible event handler to be called must therefore appear in the
    // interface of TOP. Events are dispatched by using this operator on a
    // Machine object (e.g. "machine->event()").
    template<typename TOP>
    class Machine {
    public:
        ////////////////////////////////////////////////////////////////////////////////
        // This class performs an action in its destructor after an event
        // handler has finished. Comparable to an After Advice in AOP.
        struct AfterAdvice {
            AfterAdvice(Machine<TOP> * m) : myMachine(m) {}

            // Event handler has finished execution. Execute pending transitions now.
            ~AfterAdvice() { myMachine->rattleOn(); }

            // this arrow operator finally dispatches to TOP interface.
            TOP * operator->() {
                return myMachine->myCurrentState->instance();
            }

        private:
            Machine<TOP> * myMachine;
        };

        // Constructor takes an optional preinitialized Box parameter for its
        // initial (top) state.
        Machine(typename TOP::Box * box = 0)
            : myCurrentState(0)
            , myPendingState(0)
            , myPendingBox(0)
            , myPendingHistory(false)
            , myStates(20, (StateInfo<TOP> *) 0)
        {
            // Compile time check: TOP must directly derive from Root<TOP>
            Root<TOP> * p = 0;
            typename TOP::SUPER *p2 = 0;
            p = p2;
            p2 = p;

            start(box);
        }

        ~Machine() {
            shutdown();
        }

        // Don't return pointer to interface right now: we need to know when the
        // event handler has finished; return an AfterAdvice object instead:
        // it allows us to perform actions after access.
        AfterAdvice operator->() {
            assert(myCurrentState);
            assert(!myPendingState);

            _TRC2(myCurrentState->name(), "Dispatching event");

            // We need to know when the event handler has finished.
            return AfterAdvice(this);
        }

        // Allow (const) access to top state's box (for state data extraction).
        const typename TOP::Box & box() const {
            assert(myCurrentState);
            return myCurrentState->instance()->TOP::box();
        }

    protected:
        // for getInfo
        //template<typename C, typename P>
        //friend class Link;

        // for getInfo
        friend class Root<TOP>;

        // for rattleOn
        friend struct AfterAdvice;

        // for setPendingState
        //template<class S>
        //friend struct SubStateInfo;

        // Starts the machine. Will make it go into top state.
        // Optional parameter "box" is a preinitialized state box for the top
        // state.
        void start(typename TOP::Box * box) {
            _TRC1("Starting Machine");

            myCurrentState = Root<TOP>::_getInfo(this);
            setState(TOP::_getInfo(this), false, box);
        }

        // Shuts machine down. Will exit any states and free all allocated
        // resources.
        void shutdown() {
            assert(!myPendingState);

            _TRC1("Shutting down Machine");

            // Performs exit actions.
            setState(Root<TOP>::_getInfo(this), false, 0);

            myCurrentState = 0;

            // Free all StateInfo objects now.
            typedef typename std::vector<StateInfo<TOP> *>::iterator iter;
            iter end = myStates.end();
            for (iter it = myStates.begin(); it != end; ++it) {
                delete *it;
            }
        }

        // Transition to new state (with optional preinitialized box).
        void setStateTest(Key key, bool history = true, void * box = 0) {
            StateInfo<TOP> * info = FactoryInitializer<TOP>::getFactories()[key](this);
            setPendingState(info, history, box);
            rattleOn();
        }

        // Transition to new state (with optional preinitialized box).
        void setState(StateInfo<TOP> * info, bool history, void * box) {
            setPendingState(info, history, box);
            rattleOn();
        }

    public:
        // Prepare transition to new state (performed on call to "rattleOn").
        // There can be only one state transition pending (asserts otherwise)!
        // "box" is an optional preinitialized state box for the new state.
        void setPendingState(StateInfo<TOP> * info, bool history, void * box) {
            assert( (!myPendingState || myPendingState == info)
                    && "There is already a state transition pending!");

            myPendingState = info;
            myPendingBox = box;
            myPendingHistory = history;
        }

    protected:
        // Performs pending state transition.
        void rattleOn() {
            // Loop here because init actions might change state again.
            while (myPendingState) {
                _TRC3(myCurrentState->name(), "Transition to", myPendingState->name());

                // Perform exit actions (which exactly depends on new state).
                myCurrentState->exit(myPendingState);

                // Store history information for previous state now.
                // Previous state will be used for deep history.
                myCurrentState->setHistorySuper(myCurrentState);

                if (myPendingBox)
                    myPendingState->setBox(myPendingBox);

                // Perform entry actions (which exactly depends on previous state).
                myPendingState->entry(myCurrentState);

                // State transition complete.
                myCurrentState = myPendingState;
                myPendingState = 0;
                myPendingBox = 0;
                bool history = myPendingHistory;
                myPendingHistory = false;

                // "init" may change state again.
                myCurrentState->init(history);

                assert("Init may only transition to proper substates" &&
                       (!myPendingState ||
                        (myPendingState->isChild(myCurrentState)
                         && (myCurrentState != myPendingState))));
            }
        }

    public:
        // Get StateInfo object for key.
        StateInfo<TOP> * & getInfo(Key name) {
            if (name >= myStates.size())
                myStates.resize(name + 10, (StateInfo<TOP> *) 0);

            return myStates[name];
        }

    private:
        // Current state of Machine object.
        StateInfo<TOP> * myCurrentState;

        // Information about pending state transition.
        StateInfo<TOP> * myPendingState;
        void * myPendingBox;
        bool myPendingHistory;

        // StateInfo objects.
        std::vector<StateInfo<TOP> *> myStates;
    };

} // namespace Macho

#endif
